@@ -1,17 +1,31 @@
- 
 /*
-
-    poedit, a wxWindows i18n catalogs editor
-
-    ---------------
-      catalog.cpp
-    
-      Translations catalog
-    
-      (c) Vaclav Slavik, 1999-2004
-
-*/
-
+ *  This file is part of poEdit (http://www.poedit.org)
+ *
+ *  Copyright (C) 1999-2005 Vaclav Slavik
+ *
+ *  Permission is hereby granted, free of charge, to any person obtaining a
+ *  copy of this software and associated documentation files (the "Software"),
+ *  to deal in the Software without restriction, including without limitation
+ *  the rights to use, copy, modify, merge, publish, distribute, sublicense,
+ *  and/or sell copies of the Software, and to permit persons to whom the
+ *  Software is furnished to do so, subject to the following conditions:
+ *
+ *  The above copyright notice and this permission notice shall be included in
+ *  all copies or substantial portions of the Software.
+ *
+ *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+ *  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+ *  DEALINGS IN THE SOFTWARE. 
+ *
+ *  $Id: catalog.cpp,v 1.84 2005/06/19 16:43:20 vaclavslavik Exp $
+ *
+ *  Translations catalog
+ *
+ */
 
 #include <wx/wxprec.h>
 
@@ -37,8 +51,9 @@
 
 #include "wx/arrimpl.cpp"
 WX_DEFINE_OBJARRAY(CatalogDataArray)
+WX_DEFINE_OBJARRAY(CatalogDeletedDataArray)
+
 
-    
 // ----------------------------------------------------------------------
 // Textfile processing utilities:
 // ----------------------------------------------------------------------
@@ -48,7 +63,7 @@ WX_DEFINE_OBJARRAY(CatalogDataArray)
 static wxString ReadTextLine(wxTextFile* f, wxMBConv *conv)
 {
     wxString s;
-    
+
     while (s.IsEmpty())
     {
         if (f->Eof()) return wxEmptyString;
@@ -56,7 +71,15 @@ static wxString ReadTextLine(wxTextFile* f, wxMBConv *conv)
     }
 #if !wxUSE_UNICODE
     if (conv)
-        s = wxString(s.wc_str(*conv), wxConvUTF8);
+    {
+        wxString s2 = wxString(s.wc_str(*conv), wxConvUTF8);
+        if (s2.empty() && !s.empty())
+        {
+            wxLogError(_("Error while loading file '%s': line %u is corrupted."),
+                       f->GetName(), f->GetCurrentLine());
+        }
+        return s2;
+    }
 #endif
     return s;
 }
@@ -68,7 +91,7 @@ static wxString ReadTextLine(wxTextFile* f, wxMBConv *conv)
 static bool ReadParam(const wxString& input, const wxString& pattern, wxString& output)
 {
     if (input.Length() < pattern.Length()) return false;
-    
+
     for (unsigned i = 0; i < pattern.Length(); i++)
         if (input[i] != pattern[i]) return false;
     output = input.Mid(pattern.Length()).Strip(wxString::trailing);
@@ -87,8 +110,88 @@ static bool ReadParamIfNotSet(const wxString& input,
     }
     return false;
 }
-        
-            
+
+
+#if wxUSE_UNICODE
+// Checks if the file was loaded correctly, i.e. that non-empty lines
+// ended up non-empty in memory, after doing charset conversion in
+// wxTextFile. This detects for example files that claim they are in UTF-8
+// while in fact they are not.
+bool VerifyFileCharset(const wxTextFile& f, const wxString& filename,
+                       const wxString& charset)
+{
+    wxTextFile f2;
+
+    if (!f2.Open(filename, wxConvISO8859_1))
+        return false;
+
+    if (f.GetLineCount() != f2.GetLineCount())
+    {
+        wxLogError(_("%i lines of file '%s' were not loaded correctly."),
+                   (int)f2.GetLineCount() - (int)f.GetLineCount(),
+                   filename.c_str());
+        return false;
+    }
+
+    bool ok = true;
+    size_t cnt = f.GetLineCount();
+    for (size_t i = 0; i < cnt; i++)
+    {
+        if (f[i].empty() && !f2[i].empty()) // wxMBConv conversion failed
+        {
+            wxLogError(
+                _("Line %u of file '%s' is corrupted (not valid %s data)."),
+                i, filename.c_str(), charset.c_str());
+            ok = false;
+        }
+    }
+
+    return ok;
+}
+#endif
+
+
+// converts \n into newline character and \\ into \:
+static wxString UnescapeCEscapes(const wxString& str)
+{
+    wxString out;
+    size_t len = str.size();
+    size_t i;
+
+    if ( len == 0 )
+        return str;
+
+    for (i = 0; i < len-1; i++)
+    {
+        if (str[i] == _T('\\'))
+        {
+            switch (str[i+1])
+            {
+                case _T('n'):
+                    out << _T('\n');
+                    i++;
+                    break;
+                case _T('\\'):
+                    out << _T('\\');
+                    i++;
+                    break;
+                default:
+                    out << _T('\\');
+            }
+        }
+        else
+        {
+            out << str[i];
+        }
+    }
+
+    // last character:
+    if (i < len)
+        out << str[i];
+
+    return out;
+}
+
 
 // ----------------------------------------------------------------------
 // Catalog::HeaderData
@@ -97,8 +200,7 @@ static bool ReadParamIfNotSet(const wxString& input,
 void Catalog::HeaderData::FromString(const wxString& str)
 {
     wxString hdr(str);
-    hdr.Replace(_T("\\n"), _T("\n"));
-    hdr.Replace(_T("\\\\"), _T("\\"));
+    hdr = UnescapeCEscapes(hdr);
     wxStringTokenizer tkn(hdr, _T("\n"));
     wxString ln;
 
@@ -107,7 +209,7 @@ void Catalog::HeaderData::FromString(const wxString& str)
     while (tkn.HasMoreTokens())
     {
         ln = tkn.GetNextToken();
-        size_t pos = ln.find(_T(": "));
+        size_t pos = ln.find(_T(':'));
         if (pos == wxString::npos)
         {
             wxLogError(_("Malformed header: '%s'"), ln.c_str());
@@ -115,9 +217,11 @@ void Catalog::HeaderData::FromString(const wxString& str)
         else
         {
             Entry en;
-            en.Key = ln.substr(0, pos);
-            en.Value = ln.substr(pos + 2);
+            en.Key = wxString(ln.substr(0, pos)).Strip(wxString::both);
+            en.Value = wxString(ln.substr(pos + 1)).Strip(wxString::both);
             m_entries.push_back(en);
+            wxLogTrace(_T("poedit.header"),
+                       _T("%s='%s'"), en.Key.c_str(), en.Value.c_str());
         }
     }
 
@@ -130,7 +234,7 @@ wxString Catalog::HeaderData::ToString(const wxString& line_delim)
 
     wxString hdr;
     for (std::vector<Entry>::const_iterator i = m_entries.begin();
-         i != m_entries.end(); ++i)
+         i != m_entries.end(); i++)
     {
         wxString v(i->Value);
         v.Replace(_T("\\"), _T("\\\\"));
@@ -138,30 +242,30 @@ wxString Catalog::HeaderData::ToString(const wxString& line_delim)
     }
     return hdr;
 }
-            
+
 void Catalog::HeaderData::UpdateDict()
 {
     SetHeader(_T("Project-Id-Version"), Project);
     SetHeader(_T("POT-Creation-Date"), CreationDate);
     SetHeader(_T("PO-Revision-Date"), RevisionDate);
-    
+
     if (TranslatorEmail.empty())
         SetHeader(_T("Last-Translator"), Translator);
     else
         SetHeader(_T("Last-Translator"),
                   Translator + _T(" <") + TranslatorEmail + _T(">"));
-    
+
     if (TeamEmail.empty())
         SetHeader(_T("Language-Team"), Team);
     else
         SetHeader(_T("Language-Team"),
                   Team + _T(" <") + TeamEmail + _T(">"));
-    
+
     SetHeader(_T("MIME-Version"), _T("1.0"));
     SetHeader(_T("Content-Type"), _T("text/plain; charset=") + Charset);
     SetHeader(_T("Content-Transfer-Encoding"), _T("8bit"));
-    
-   
+
+
     // Set extended information:
 
     SetHeaderNotEmpty(_T("X-Poedit-Language"), Language);
@@ -172,14 +276,28 @@ void Catalog::HeaderData::UpdateDict()
     {
         wxString kw;
         for (size_t i = 0; i < Keywords.GetCount(); i++)
-            kw += Keywords[i] + _T(',');
+            kw += Keywords[i] + _T(';');
         kw.RemoveLast();
-        SetHeader(_T("X-Poedit-Keywords"), kw);
+        SetHeader(_T("X-Poedit-KeywordsList"), kw);
     }
-    
+
+    int i;
+    bool noBookmarkSet = true;
+    wxString bk;
+    for (i = 0; i < BOOKMARK_LAST ; i++)
+    {
+        noBookmarkSet = noBookmarkSet && (Bookmarks[i] == NO_BOOKMARK);
+        bk += wxString() << Bookmarks[i] << _T(',');
+    }
+    bk.RemoveLast();
+    if (noBookmarkSet)
+        DeleteHeader(_T("X-Poedit-Bookmarks"));
+    else
+        SetHeader(_T("X-Poedit-Bookmarks"), bk);
+
     SetHeaderNotEmpty(_T("X-Poedit-Basepath"), BasePath);
 
-    int i = 0;
+    i = 0;
     wxString path;
     while (true)
     {
@@ -189,7 +307,7 @@ void Catalog::HeaderData::UpdateDict()
         DeleteHeader(path);
         i++;
     }
-        
+
     for (i = 0; i < SearchPaths.GetCount(); i++)
     {
         path.Printf(_T("X-Poedit-SearchPath-%i"), i);
@@ -200,11 +318,11 @@ void Catalog::HeaderData::UpdateDict()
 void Catalog::HeaderData::ParseDict()
 {
     wxString dummy;
-    
+
     Project = GetHeader(_T("Project-Id-Version"));
     CreationDate = GetHeader(_T("POT-Creation-Date"));
     RevisionDate = GetHeader(_T("PO-Revision-Date"));
-           
+
     dummy = GetHeader(_T("Last-Translator"));
     if (!dummy.empty())
     {
@@ -225,7 +343,7 @@ void Catalog::HeaderData::ParseDict()
     if (!dummy.empty())
     {
         wxStringTokenizer tkn(dummy, _T("<>"));
-        if (tkn.CountTokens() != 2) 
+        if (tkn.CountTokens() != 2)
         {
             Team = dummy;
             TeamEmail = wxEmptyString;
@@ -236,12 +354,12 @@ void Catalog::HeaderData::ParseDict()
             TeamEmail = tkn.GetNextToken();
         }
     }
-        
+
     wxString ctype = GetHeader(_T("Content-Type"));
     if (!ReadParam(ctype, _T("text/plain; charset="), Charset))
         Charset = _T("iso-8859-1");
-    
-    
+
+
     // Parse extended information:
 
     Language = GetHeader(_T("X-Poedit-Language"));
@@ -250,15 +368,51 @@ void Catalog::HeaderData::ParseDict()
     BasePath = GetHeader(_T("X-Poedit-Basepath"));
 
     Keywords.Clear();
-    wxString kw = GetHeader(_T("X-Poedit-Keywords"));
+    wxString kw = GetHeader(_T("X-Poedit-KeywordsList"));
     if (!kw.empty())
     {
-        wxStringTokenizer tkn(kw, _T(","));
+        wxStringTokenizer tkn(kw, _T(";"));
         while (tkn.HasMoreTokens())
             Keywords.Add(tkn.GetNextToken());
     }
+    else
+    {
+        // try backward-compatibility version X-Poedit-Keywords. The difference
+        // is the separator used, see
+        // http://sourceforge.net/tracker/index.php?func=detail&aid=1206579&group_id=27043&atid=389153
 
-    int i = 0;
+        wxString kw = GetHeader(_T("X-Poedit-Keywords"));
+        if (!kw.empty())
+        {
+            wxStringTokenizer tkn(kw, _T(","));
+            while (tkn.HasMoreTokens())
+                Keywords.Add(tkn.GetNextToken());
+
+            // and remove it, it's not for newer versions:
+            DeleteHeader(_T("X-Poedit-Keywords"));
+        }
+    }
+
+    int i;
+    for(i = 0; i < BOOKMARK_LAST; i++)
+    {
+      Bookmarks[i] = NO_BOOKMARK;
+    }
+    wxString bk = GetHeader(_T("X-Poedit-Bookmarks"));
+    if (!bk.empty())
+    {
+        wxStringTokenizer tkn(bk, _T(","));
+        i=0;
+        long int val;
+        while (tkn.HasMoreTokens() && i<BOOKMARK_LAST)
+        {
+            tkn.GetNextToken().ToLong(&val);
+            Bookmarks[i] = val;
+            i++;
+        }
+    }
+
+    i = 0;
     wxString path;
     SearchPaths.Clear();
     while (true)
@@ -315,10 +469,10 @@ void Catalog::HeaderData::DeleteHeader(const wxString& key)
     if (HasHeader(key))
     {
         Entries enew;
-        
+
         size_t size = m_entries.size();
         for (Entries::const_iterator i = m_entries.begin();
-                i != m_entries.end(); ++i)
+                i != m_entries.end(); i++)
         {
             if (i->Key != key)
                 enew.push_back(*i);
@@ -344,7 +498,7 @@ Catalog::HeaderData::Find(const wxString& key) const
 // ----------------------------------------------------------------------
 // Parsers
 // ----------------------------------------------------------------------
-        
+
 void CatalogParser::Parse()
 {
     if (m_textFile->GetLineCount() == 0) return;
@@ -354,23 +508,17 @@ void CatalogParser::Parse()
     wxArrayString mrefs, mautocomments, mtranslations;
     bool has_plural = false;
     unsigned mlinenum;
-    
+
     line = m_textFile->GetFirstLine();
     if (line.IsEmpty()) line = ReadTextLine(m_textFile, m_conv);
 
     while (!line.IsEmpty())
     {
-        // ignore empty special tags:
-        while (line == _T("#,") || line == _T("#:") || line == _T("#."))
+        // ignore empty special tags (except for automatic comments which we
+        // DO want to preserve):
+        while (line == _T("#,") || line == _T("#:"))
             line = ReadTextLine(m_textFile, m_conv);
 
-        // auto comments:
-        if (ReadParam(line, _T("#. "), dummy))
-        {
-            mautocomments.Add(dummy);
-            line = ReadTextLine(m_textFile, m_conv);
-        }
- 
         // flags:
         // Can't we have more than one flag, now only the last is kept ...
         if (ReadParam(line, _T("#, "), dummy))
@@ -378,9 +526,16 @@ void CatalogParser::Parse()
             mflags = _T("#, ") + dummy;
             line = ReadTextLine(m_textFile, m_conv);
         }
-        
+
+        // auto comments:
+        if (ReadParam(line, _T("#. "), dummy) || ReadParam(line, _T("#."), dummy)) // second one to account for empty auto comments
+        {
+            mautocomments.Add(dummy);
+            line = ReadTextLine(m_textFile, m_conv);
+        }
+
         // references:
-        if (ReadParam(line, _T("#: "), dummy))
+        else if (ReadParam(line, _T("#: "), dummy))
         {
             // A line may contain several references, separated by white-space.
             // Each reference is in the form "path_name:line_number"
@@ -398,7 +553,7 @@ void CatalogParser::Parse()
 
             line = ReadTextLine(m_textFile, m_conv);
         }
-        
+
         // msgid:
         else if (ReadParam(line, _T("msgid \""), dummy) ||
                  ReadParam(line, _T("msgid\t\""), dummy))
@@ -440,7 +595,7 @@ void CatalogParser::Parse()
         {
             if (has_plural)
                 wxLogError(_("Broken catalog file: singular form msgstr used together with msgid_plural"));
-            
+
             wxString str = dummy.RemoveLast();
             while (!(line = ReadTextLine(m_textFile, m_conv)).IsEmpty())
             {
@@ -472,15 +627,15 @@ void CatalogParser::Parse()
         {
             if (!has_plural)
                 wxLogError(_("Broken catalog file: plural form msgstr used without msgid_plural"));
-            
+
             wxString idx = dummy.BeforeFirst(_T(']'));
             wxString label = _T("msgstr[") + idx + _T("]");
-        
+
             while (ReadParam(line, label + _T(" \""), dummy) ||
                    ReadParam(line, label + _T("\t\""), dummy))
             {
                 wxString str = dummy.RemoveLast();
-                
+
                 while (!(line=ReadTextLine(m_textFile, m_conv)).IsEmpty())
                 {
                     if (line[0u] == _T('\t'))
@@ -513,13 +668,41 @@ void CatalogParser::Parse()
             mautocomments.Clear();
             mtranslations.Clear();
         }
-        
+
+        // deleted lines:
+        else if (ReadParam(line, _T("#~ "), dummy))
+        {
+            wxArrayString deletedLines;
+            deletedLines.Add(line);
+            mlinenum = m_textFile->GetCurrentLine() + 1;
+            while (!(line = ReadTextLine(m_textFile, m_conv)).IsEmpty())
+            {
+                // if line does not start with "#~ " anymore, stop reading
+                if (!ReadParam(line, _T("#~ "), dummy))
+                    break;
+
+                deletedLines.Add(line);
+            }
+            if (!OnDeletedEntry(deletedLines,
+                                mflags, mrefs, mcomment, mautocomments, mlinenum))
+            {
+                return;
+            }
+
+            mcomment = mstr = msgid_plural = mflags = wxEmptyString;
+            has_plural = false;
+            mrefs.Clear();
+            mautocomments.Clear();
+            mtranslations.Clear();
+        }
+
         // comment:
         else if (line[0u] == _T('#'))
         {
             bool readNewLine = false;
 
-            while (!line.IsEmpty() && line[0u] == _T('#') &&
+            while (!line.IsEmpty() &&
+                    line[0u] == _T('#') &&
                    (line.Length() < 2 || (line[1u] != _T(',') && line[1u] != _T(':') && line[1u] != _T('.') )))
             {
                 mcomment << line << _T('\n');
@@ -530,7 +713,7 @@ void CatalogParser::Parse()
             if (!readNewLine)
                 line = ReadTextLine(m_textFile, m_conv);
         }
-        
+
         else
             line = ReadTextLine(m_textFile, m_conv);
     }
@@ -541,13 +724,13 @@ void CatalogParser::Parse()
 class CharsetInfoFinder : public CatalogParser
 {
     public:
-        CharsetInfoFinder(wxTextFile *f, wxMBConv *conv) 
+        CharsetInfoFinder(wxTextFile *f, wxMBConv *conv)
                 : CatalogParser(f, conv), m_charset(_T("iso-8859-1")) {}
         wxString GetCharset() const { return m_charset; }
 
     protected:
         wxString m_charset;
-    
+
         virtual bool OnEntry(const wxString& msgid,
                              const wxString& msgid_plural,
                              bool has_plural,
@@ -557,6 +740,7 @@ class CharsetInfoFinder : public CatalogParser
                              const wxString& comment,
                              const wxArrayString& autocomments,
                              unsigned lineNumber);
+
 };
 
 bool CharsetInfoFinder::OnEntry(const wxString& msgid,
@@ -566,7 +750,7 @@ bool CharsetInfoFinder::OnEntry(const wxString& msgid,
                                 const wxString& flags,
                                 const wxArrayString& references,
                                 const wxString& comment,
-                                const wxArrayString& autocomments, 
+                                const wxArrayString& autocomments,
                                 unsigned lineNumber)
 {
     if (msgid.IsEmpty())
@@ -592,7 +776,7 @@ class LoadParser : public CatalogParser
 
     protected:
         Catalog *m_catalog;
-    
+
         virtual bool OnEntry(const wxString& msgid,
                              const wxString& msgid_plural,
                              bool has_plural,
@@ -602,6 +786,13 @@ class LoadParser : public CatalogParser
                              const wxString& comment,
                              const wxArrayString& autocomments,
                              unsigned lineNumber);
+
+        virtual bool OnDeletedEntry(const wxArrayString& deletedLines,
+                                    const wxString& flags,
+                                    const wxArrayString& references,
+                                    const wxString& comment,
+                                    const wxArrayString& autocomments,
+                                    unsigned lineNumber);
 };
 
 
@@ -612,7 +803,7 @@ bool LoadParser::OnEntry(const wxString& msgid,
                          const wxString& flags,
                          const wxArrayString& references,
                          const wxString& comment,
-                         const wxArrayString& autocomments, 
+                         const wxArrayString& autocomments,
                          unsigned lineNumber)
 {
     if (msgid.IsEmpty())
@@ -640,6 +831,25 @@ bool LoadParser::OnEntry(const wxString& msgid,
     return true;
 }
 
+bool LoadParser::OnDeletedEntry(const wxArrayString& deletedLines,
+                                const wxString& flags,
+                                const wxArrayString& references,
+                                const wxString& comment,
+                                const wxArrayString& autocomments,
+                                unsigned lineNumber)
+{
+    CatalogDeletedData *d = new CatalogDeletedData(wxArrayString());
+    if (!flags.IsEmpty()) d->SetFlags(flags);
+    d->SetDeletedLines(deletedLines);
+    d->SetComment(comment);
+    d->SetLineNumber(lineNumber);
+    for (size_t i = 0; i < autocomments.GetCount(); i++)
+      d->AddAutoComments(autocomments[i]);
+    m_catalog->AddDeletedItem(d);
+
+    return true;
+}
+
 
 
 // ----------------------------------------------------------------------
@@ -652,10 +862,14 @@ Catalog::Catalog()
     m_count = 0;
     m_isOk = true;
     m_header.BasePath = wxEmptyString;
+    for(int i = BOOKMARK_0; i < BOOKMARK_LAST; i++)
+    {
+        m_header.Bookmarks[i] = -1;
+    }
 }
 
 
-Catalog::~Catalog() 
+Catalog::~Catalog()
 {
     Clear();
     delete m_data;
@@ -686,7 +900,7 @@ static wxString GetCurrentTimeRFC822()
 void Catalog::CreateNewHeader()
 {
     HeaderData &dt = Header();
-    
+
     dt.CreationDate = GetCurrentTimeRFC822();
     dt.RevisionDate = dt.CreationDate;
 
@@ -716,18 +930,18 @@ bool Catalog::Load(const wxString& po_file)
     wxTextFile f;
 
     Clear();
-    delete m_data; m_data = NULL;    
+    delete m_data; m_data = NULL;
     m_count = 0;
     m_isOk = false;
     m_fileName = po_file;
     m_header.BasePath = wxEmptyString;
-    
+
     /* Load the .po file: */
-        
+
     m_data = new wxHashTable(wxKEY_STRING);
 
     if (!f.Open(po_file)) return false;
-    
+
     CharsetInfoFinder charsetFinder(&f, &wxConvISO8859_1);
     charsetFinder.Parse();
     m_header.Charset = charsetFinder.GetCharset();
@@ -736,17 +950,32 @@ bool Catalog::Load(const wxString& po_file)
     wxCSConv encConv(m_header.Charset);
     if (!f.Open(po_file, encConv)) return false;
 
+#if wxUSE_UNICODE
+    if (!VerifyFileCharset(f, po_file, m_header.Charset))
+    {
+        wxLogError(_("There were errors when loading the catalog. Some data may be missing or corrupted as the result."));
+    }
+#endif
+
     LoadParser parser(this, &f, &encConv);
     parser.Parse();
 
+    // now that the catalog is loaded, update its items with the bookmarks
+    for(int i = BOOKMARK_0; i < BOOKMARK_LAST; i++)
+    {
+        if (m_header.Bookmarks[i] != -1 && m_header.Bookmarks[i] < m_dataArray.GetCount())
+            m_dataArray[m_header.Bookmarks[i]].SetBookmark(static_cast<Bookmark>(i));
+    }
+
+
     m_isOk = true;
-    
+
     f.Close();
 
     /* Load extended information from .po.poedit file, if present:
        (NB: this is deprecated, poedit >= 1.3.0 stores the data in
             .po file's header as X-Poedit-Foo) */
-    
+
     if (wxFileExists(po_file + _T(".poedit")) &&
         f.Open(po_file + _T(".poedit")))
     {
@@ -799,10 +1028,10 @@ bool Catalog::Load(const wxString& po_file)
 
         f.Close();
     }
-    
+
     return true;
 }
-        
+
 void Catalog::AddItem(CatalogData *data)
 {
     m_data->Put(data->GetString(), data);
@@ -810,13 +1039,53 @@ void Catalog::AddItem(CatalogData *data)
     m_count++;
 }
 
+void Catalog::AddDeletedItem(CatalogDeletedData *data)
+{
+    m_deletedItemsArray.Add(data);
+}
+
+bool Catalog::HasDeletedItems()
+{
+    return !m_deletedItemsArray.IsEmpty();
+}
+
+void Catalog::RemoveDeletedItems()
+{
+    m_deletedItemsArray.Empty();
+}
+
 void Catalog::Clear()
 {
-    delete m_data; 
+    delete m_data;
     m_dataArray.Empty();
+    m_deletedItemsArray.Empty();
     m_isOk = true;
     m_count = 0;
     m_data = new wxHashTable(wxKEY_STRING);
+    for(int i = BOOKMARK_0; i < BOOKMARK_LAST; i++)
+    {
+        m_header.Bookmarks[i] = -1;
+    }
+}
+
+int Catalog::SetBookmark(int id, Bookmark bookmark)
+{
+    int result = (bookmark==NO_BOOKMARK)?-1:m_header.Bookmarks[bookmark];
+
+    // unset previous bookmarks, if any
+    Bookmark bk = m_dataArray[id].GetBookmark();
+    if (bk != NO_BOOKMARK)
+        m_header.Bookmarks[bk] = -1;
+    if (result > -1)
+        m_dataArray[result].SetBookmark(NO_BOOKMARK);
+
+    // set new bookmark
+    m_dataArray[id].SetBookmark(bookmark);
+    if (bookmark != NO_BOOKMARK)
+        m_header.Bookmarks[bookmark] = id;
+
+    // return id of previous item for that bookmark
+    return result;
 }
 
 
@@ -855,19 +1124,19 @@ static bool CanEncodeToCharset(Catalog& catalog, const wxString& charset)
 {
     if (charset == _T("utf-8") || charset == _T("UTF-8"))
         return true;
-    
+
     wxCSConv conv(charset);
 
     catalog.Header().UpdateDict();
     const Catalog::HeaderData::Entries& hdr(catalog.Header().GetAllHeaders());
-    
+
     for (Catalog::HeaderData::Entries::const_iterator i = hdr.begin();
-            i != hdr.end(); ++i)
+            i != hdr.end(); i++)
     {
         if (!CanEncodeStringToCharset(i->Value, conv))
             return false;
     }
-    
+
     size_t cnt = catalog.GetCount();
     for (size_t i = 0; i < cnt; i++)
     {
@@ -907,7 +1176,7 @@ static wxString FormatStringForFile(const wxString& text)
     wxString s;
     unsigned n_cnt = 0;
     int len = text.length();
-    
+
     s.Alloc(len + 16);
     // Scan the string up to len-2 because we don't want to account for the
     // very last \n on the line:
@@ -930,7 +1199,7 @@ static wxString FormatStringForFile(const wxString& text)
     // ...and add not yet processed characters to the string...
     for (; i < len; i++)
         s << text[i];
-    
+
     if (n_cnt >= 1)
         return _T("\"\n\"") + s;
     else
@@ -950,17 +1219,25 @@ static inline wxString convertUtf8ToCharset(const wxString& s, wxMBConv *conv)
 bool Catalog::Save(const wxString& po_file, bool save_mo)
 {
     CatalogData *data;
+    CatalogDeletedData *deletedItem;
     wxString dummy;
     size_t i;
     int j;
     wxTextFileType crlfOrig, crlf;
     bool crlfPreserve;
     wxTextFile f;
-    
+
+    if ( wxFileExists(po_file) && !wxFile::Access(po_file, wxFile::write) )
+    {
+        wxLogError(_("File '%s' is read-only and cannot be saved.\nPlease save it under different name."),
+                   po_file.c_str());
+        return false;
+    }
+
     GetCRLFBehaviour(crlfOrig, crlfPreserve);
 
     /* Update information about last modification time: */
- 
+
     m_header.RevisionDate = GetCurrentTimeRFC822();
 
     /* Detect CRLF format: */
@@ -981,9 +1258,9 @@ bool Catalog::Save(const wxString& po_file, bool save_mo)
     /* Save .po file: */
 
     wxString charset(m_header.Charset);
-    if (!charset)
+    if (!charset || charset == _T("CHARSET"))
         charset = _T("utf-8");
-        
+
     if (!CanEncodeToCharset(*this, charset))
     {
         wxString msg;
@@ -993,13 +1270,13 @@ bool Catalog::Save(const wxString& po_file, bool save_mo)
         charset = _T("utf-8");
     }
     m_header.Charset = charset;
-   
+
     if (!wxFileExists(po_file) || !f.Open(po_file))
         if (!f.Create(po_file))
             return false;
     for (j = f.GetLineCount() - 1; j >= 0; j--)
         f.RemoveLine(j);
-   
+
 
     wxCSConv *encConv;
 #if wxUSE_UNICODE
@@ -1025,7 +1302,12 @@ bool Catalog::Save(const wxString& po_file, bool save_mo)
         data = &(m_dataArray[i]);
         SaveMultiLines(f, convertUtf8ToCharset(data->GetComment(), encConv));
         for (unsigned i = 0; i < data->GetAutoComments().GetCount(); i++)
-            f.AddLine(_T("#. ") + data->GetAutoComments()[i]);
+        {
+            if (data->GetAutoComments()[i].empty())
+              f.AddLine(_T("#."));
+            else
+              f.AddLine(_T("#. ") + data->GetAutoComments()[i]);
+        }
         for (unsigned i = 0; i < data->GetReferences().GetCount(); i++)
             f.AddLine(_T("#: ") + data->GetReferences()[i]);
         dummy = data->GetFlags();
@@ -1045,7 +1327,7 @@ bool Catalog::Save(const wxString& po_file, bool save_mo)
             for (size_t i = 0; i < data->GetNumberOfTranslations(); i++)
             {
                 dummy = convertUtf8ToCharset(
-                            FormatStringForFile(data->GetTranslation(i)), 
+                            FormatStringForFile(data->GetTranslation(i)),
                             encConv);
                 wxString hdr = wxString::Format(_T("msgstr[%u] \""), i);
                 SaveMultiLines(f, hdr + dummy + _T("\""));
@@ -1060,7 +1342,28 @@ bool Catalog::Save(const wxString& po_file, bool save_mo)
         }
         f.AddLine(wxEmptyString);
     }
-    
+
+    // Write back deleted items in the file so that they're not lost
+    for (i = 0; i < m_deletedItemsArray.GetCount(); i++)
+    {
+        deletedItem = &(m_deletedItemsArray[i]);
+        SaveMultiLines(f, convertUtf8ToCharset(deletedItem->GetComment(), encConv));
+        for (unsigned i = 0; i < deletedItem->GetAutoComments().GetCount(); i++)
+            f.AddLine(_T("#. ") + deletedItem->GetAutoComments()[i]);
+        for (unsigned i = 0; i < deletedItem->GetReferences().GetCount(); i++)
+            f.AddLine(_T("#: ") + deletedItem->GetReferences()[i]);
+        dummy = deletedItem->GetFlags();
+        if (!dummy.IsEmpty())
+            f.AddLine(dummy);
+        deletedItem->SetLineNumber(f.GetLineCount()+1);
+
+        for (size_t j = 0; j < deletedItem->GetDeletedLines().GetCount(); j++)
+            f.AddLine(convertUtf8ToCharset(deletedItem->GetDeletedLines()[j], encConv));
+
+        f.AddLine(wxEmptyString);
+    }
+
+
 #if wxUSE_UNICODE
     f.Write(crlf, *encConv);
 #else
@@ -1074,22 +1377,22 @@ bool Catalog::Save(const wxString& po_file, bool save_mo)
     {
         wxRemoveFile(po_file + _T(".poedit"));
     }
-    
-    delete encConv;    
+
+    delete encConv;
 
     /* If the user wants it, compile .mo file right now: */
-    
+
     if (save_mo && wxConfig::Get()->Read(_T("compile_mo"), true))
-        ExecuteGettext(_T("msgfmt -c -o \"") + 
+        ExecuteGettext(_T("msgfmt -c -o \"") +
                        po_file.BeforeLast(_T('.')) + _T(".mo\" \"") + po_file + _T("\""));
 
     m_fileName = po_file;
-    
+
     return true;
 }
 
 
-bool Catalog::Update()
+bool Catalog::Update(bool summary)
 {
     if (!m_isOk) return false;
 
@@ -1097,23 +1400,23 @@ bool Catalog::Update()
     pinfo.SetTitle(_("Updating catalog..."));
 
     wxString cwd = wxGetCwd();
-    if (m_fileName != wxEmptyString) 
+    if (m_fileName != wxEmptyString)
     {
         wxString path;
-        
+
         if (wxIsAbsolutePath(m_header.BasePath))
             path = m_header.BasePath;
         else
             path = wxPathOnly(m_fileName) + _T("/") + m_header.BasePath;
-        
+
         if (wxIsAbsolutePath(path))
             wxSetWorkingDirectory(path);
         else
             wxSetWorkingDirectory(cwd + _T("/") + path);
     }
-    
+
     SourceDigger dig(&pinfo);
-    
+
     wxArrayString keywords;
     if (m_header.Keywords.IsEmpty())
     {
@@ -1126,17 +1429,17 @@ bool Catalog::Update()
     {
         WX_APPEND_ARRAY(keywords, m_header.Keywords);
     }
-    
+
     Catalog *newcat = dig.Dig(m_header.SearchPaths,
                               keywords,
                               m_header.SourceCodeCharset);
 
-    if (newcat != NULL) 
+    if (newcat != NULL)
     {
         bool succ = false;
         pinfo.UpdateMessage(_("Merging differences..."));
 
-        if (ShowMergeSummary(newcat))
+        if (!summary || ShowMergeSummary(newcat))
             succ = Merge(newcat);
         if (!succ)
         {
@@ -1144,17 +1447,17 @@ bool Catalog::Update()
             newcat = NULL;
         }
     }
-    
+
     wxSetWorkingDirectory(cwd);
-    
+
     if (newcat == NULL) return false;
-  
+
     delete newcat;
     return true;
 }
 
 
-bool Catalog::UpdateFromPOT(const wxString& pot_file)
+bool Catalog::UpdateFromPOT(const wxString& pot_file, bool summary)
 {
     if (!m_isOk) return false;
 
@@ -1166,7 +1469,7 @@ bool Catalog::UpdateFromPOT(const wxString& pot_file)
         return false;
     }
 
-    if (ShowMergeSummary(&newcat))
+    if (!summary || ShowMergeSummary(&newcat))
         return Merge(&newcat);
     else
         return false;
@@ -1176,15 +1479,21 @@ bool Catalog::UpdateFromPOT(const wxString& pot_file)
 bool Catalog::Merge(Catalog *refcat)
 {
     wxString oldname = m_fileName;
-    wxString tmp1 = wxGetTempFileName(_T("poedit"));
-    wxString tmp2 = wxGetTempFileName(_T("poedit"));
-    wxString tmp3 = wxGetTempFileName(_T("poedit"));
+
+    wxString tmpdir = wxGetTempFileName(_T("poedit"));
+    wxRemoveFile(tmpdir);
+    if (!wxMkdir(tmpdir, 0700))
+        return false;
+
+    wxString tmp1 = tmpdir + wxFILE_SEP_PATH + _T("ref.pot");
+    wxString tmp2 = tmpdir + wxFILE_SEP_PATH + _T("input.po");
+    wxString tmp3 = tmpdir + wxFILE_SEP_PATH + _T("output.po");
 
     refcat->Save(tmp1, false);
     Save(tmp2, false);
-    
+
     bool succ =
-        ExecuteGettext(_T("msgmerge --force-po -o \"") + tmp3 + _T("\" \"") + 
+        ExecuteGettext(_T("msgmerge --force-po -o \"") + tmp3 + _T("\" \"") +
                        tmp2 + _T("\" \"") + tmp1 + _T("\""));
     if (succ)
     {
@@ -1197,22 +1506,21 @@ bool Catalog::Merge(Catalog *refcat)
     wxRemoveFile(tmp1);
     wxRemoveFile(tmp2);
     wxRemoveFile(tmp3);
-    wxRemoveFile(tmp1 + _T(".poedit"));
-    wxRemoveFile(tmp2 + _T(".poedit"));
-    
+    wxRmdir(tmpdir);
+
     m_fileName = oldname;
 
     return succ;
 }
 
 
-void Catalog::GetMergeSummary(Catalog *refcat, 
+void Catalog::GetMergeSummary(Catalog *refcat,
                           wxArrayString& snew, wxArrayString& sobsolete)
 {
     snew.Empty();
     sobsolete.Empty();
     unsigned i;
-    
+
     for (i = 0; i < GetCount(); i++)
         if (refcat->FindItem((*this)[i].GetString()) == NULL)
         sobsolete.Add((*this)[i].GetString());
@@ -1221,9 +1529,9 @@ void Catalog::GetMergeSummary(Catalog *refcat,
         if (FindItem((*refcat)[i].GetString()) == NULL)
         snew.Add((*refcat)[i].GetString());
 }
-        
+
 bool Catalog::ShowMergeSummary(Catalog *refcat)
-{    
+{
     if (wxConfig::Get()->Read(_T("show_summary"), true))
     {
         wxArrayString snew, sobsolete;
@@ -1259,7 +1567,7 @@ unsigned Catalog::GetPluralFormsCount() const
                 return val;
         }
     }
-    
+
     return 0;
 }
 
@@ -1267,8 +1575,8 @@ unsigned Catalog::GetPluralFormsCount() const
 bool Catalog::Translate(const wxString& key, const wxString& translation)
 {
     CatalogData *d = FindItem(key);
-    
-    if (d == NULL) 
+
+    if (d == NULL)
         return false;
     else
     {
@@ -1281,7 +1589,7 @@ bool Catalog::Translate(const wxString& key, const wxString& translation)
 void Catalog::Append(Catalog& cat)
 {
     CatalogData *dt, *mydt;
-    
+
     for (unsigned i = 0; i < cat.GetCount(); i++)
     {
         dt = &(cat[i]);
@@ -1328,7 +1636,7 @@ void CatalogData::SetFlags(const wxString& flags)
 {
     m_isFuzzy = false;
     m_moreFlags.Empty();
-    
+
     if (flags.IsEmpty()) return;
     wxStringTokenizer tkn(flags.Mid(1), _T(" ,"), wxTOKEN_STRTOK);
     wxString s;
@@ -1346,12 +1654,12 @@ wxString CatalogData::GetFlags() const
     wxString f;
     if (m_isFuzzy) f << _T(", fuzzy");
     f << m_moreFlags;
-    if (!f.IsEmpty()) 
+    if (!f.IsEmpty())
         return _T("#") + f;
-    else 
+    else
         return wxEmptyString;
 }
-        
+
 bool CatalogData::IsInFormat(const wxString& format)
 {
     wxString lookingFor;
@@ -1367,7 +1675,7 @@ bool CatalogData::IsInFormat(const wxString& format)
 
 wxString CatalogData::GetTranslation(unsigned idx) const
 {
-    if (idx >= GetNumberOfTranslations()) 
+    if (idx >= GetNumberOfTranslations())
         return wxEmptyString;
     else
         return m_translations[idx];
@@ -1378,7 +1686,7 @@ void CatalogData::SetTranslation(const wxString &t, unsigned idx)
     while (idx >= m_translations.GetCount())
         m_translations.Add(wxEmptyString);
     m_translations[idx] = t;
-    
+
     m_validity = Val_Unknown;
     m_isTranslated = false;
     for (size_t i = 0; i < m_translations.GetCount(); i++)
@@ -1389,14 +1697,14 @@ void CatalogData::SetTranslation(const wxString &t, unsigned idx)
 void CatalogData::SetTranslations(const wxArrayString &t)
 {
     m_translations = t;
-    
+
     m_validity = Val_Unknown;
     m_isTranslated = false;
     for (size_t i = 0; i < m_translations.GetCount(); i++)
         if (!m_translations[i].IsEmpty())
             m_isTranslated = true;
 }
-       
+
 
 wxString Catalog::GetLocaleCode() const
 {
@@ -1418,7 +1726,7 @@ wxString Catalog::GetLocaleCode() const
     {
         wxString name;
         wxFileName::SplitPath(m_fileName, NULL, &name, NULL);
-        
+
         if (name.length() == 2)
         {
             if (IsKnownLanguageCode(name))
@@ -1433,6 +1741,6 @@ wxString Catalog::GetLocaleCode() const
     }
 
     wxLogTrace(_T("poedit"), _T("catalog lang is '%s'"), lang.c_str());
-    
+
     return lang;
 }
