@@ -1,16 +1,31 @@
-
 /*
-
-    poedit, a wxWindows i18n catalogs editor
-
-    ---------------
-      catalog.h
-    
-      Translations catalog
-    
-      (c) Vaclav Slavik, 1999-2004
-
-*/
+ *  This file is part of poEdit (http://www.poedit.org)
+ *
+ *  Copyright (C) 1999-2005 Vaclav Slavik
+ *
+ *  Permission is hereby granted, free of charge, to any person obtaining a
+ *  copy of this software and associated documentation files (the "Software"),
+ *  to deal in the Software without restriction, including without limitation
+ *  the rights to use, copy, modify, merge, publish, distribute, sublicense,
+ *  and/or sell copies of the Software, and to permit persons to whom the
+ *  Software is furnished to do so, subject to the following conditions:
+ *
+ *  The above copyright notice and this permission notice shall be included in
+ *  all copies or substantial portions of the Software.
+ *
+ *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+ *  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+ *  DEALINGS IN THE SOFTWARE. 
+ *
+ *  $Id: catalog.h,v 1.44 2005/06/19 16:43:20 vaclavslavik Exp $
+ *
+ *  Translations catalog
+ *
+ */
 
 #ifndef _CATALOG_H_
 #define _CATALOG_H_
@@ -24,8 +39,27 @@
 class WXDLLEXPORT wxTextFile;
 
 class CatalogData;
+class CatalogDeletedData;
 WX_DECLARE_OBJARRAY(CatalogData, CatalogDataArray);
-    
+WX_DECLARE_OBJARRAY(CatalogDeletedData, CatalogDeletedDataArray);
+
+/// The possible bookmarks for a given item
+typedef enum
+{
+    NO_BOOKMARK = -1,
+    BOOKMARK_0,
+    BOOKMARK_1,
+    BOOKMARK_2,
+    BOOKMARK_3,
+    BOOKMARK_4,
+    BOOKMARK_5,
+    BOOKMARK_6,
+    BOOKMARK_7,
+    BOOKMARK_8,
+    BOOKMARK_9,
+    BOOKMARK_LAST
+} Bookmark;
+
 /** This class stores all translations, together with filelists, references
     and other additional information. It can read .po files and save both
     .mo and .po files. Furthermore, it provides facilities for updating the
@@ -39,7 +73,7 @@ class Catalog
         {
         public:
             HeaderData() {}
-            
+
             /** Initializes the headers from string that is in msgid "" format
                 (i.e. list of key:value\n entries). */
             void FromString(const wxString& str);
@@ -47,12 +81,12 @@ class Catalog
             /** Converts the header into string representation that can be
                 directly written to .po file as msgid "". */
             wxString ToString(const wxString& line_delim = wxEmptyString);
-            
+
             /// Updates headers list from parsed values entries below
             void UpdateDict();
             /// Reverse operation to UpdateDict
             void ParseDict();
-            
+
             /// Returns value of header or empty string if missing.
             wxString GetHeader(const wxString& key) const;
 
@@ -62,28 +96,29 @@ class Catalog
             /** Sets header to given value. Overwrites old value if present,
                 appends to the end of header values otherwise. */
             void SetHeader(const wxString& key, const wxString& value);
-            
+
             /// Like SetHeader, but deletes the header if value is empty
             void SetHeaderNotEmpty(const wxString& key, const wxString& value);
 
             /// Removes given header entry
             void DeleteHeader(const wxString& key);
-    
+
             struct Entry
             {
                 wxString Key, Value;
             };
             typedef std::vector<Entry> Entries;
-            
+
             const Entries& GetAllHeaders() const { return m_entries; }
 
             // Parsed values:
-            
-            wxString Language, Country, Project, CreationDate, 
+
+            wxString Language, Country, Project, CreationDate,
                      RevisionDate, Translator, TranslatorEmail,
                      Team, TeamEmail, Charset, SourceCodeCharset;
 
             wxArrayString SearchPaths, Keywords;
+            int Bookmarks[BOOKMARK_LAST];
             wxString BasePath;
 
             wxString Comment;
@@ -111,18 +146,18 @@ class Catalog
         /** Loads catalog from .po file.
             If file named po_file ".poedit" (e.g. "cs.po.poedit") exists,
             this function loads additional information from it. .po.poedit
-            file contains parts of catalog header data that are not part 
+            file contains parts of catalog header data that are not part
             of standard .po format, namely SearchPaths, Keywords, BasePath
             and Language.
          */
         bool Load(const wxString& po_file);
 
-        /** Saves catalog to file. Creates both .po (text) and .mo (binary) 
-            version of the catalog (unless the latter was disabled in 
+        /** Saves catalog to file. Creates both .po (text) and .mo (binary)
+            version of the catalog (unless the latter was disabled in
             preferences). Calls external xmsgfmt program to generate the .mo
-            file. 
+            file.
 
-            Note that \a po_file refers to .po file, .mo file will have same 
+            Note that \a po_file refers to .po file, .mo file will have same
             name & location as .po file except for different extension.
          */
         bool Save(const wxString& po_file, bool save_mo = true);
@@ -133,20 +168,20 @@ class Catalog
         /** Updates the catalog from sources.
             \see SourceDigger, Parser, UpdateFromPOT.
          */
-        bool Update();
+        bool Update(bool summary = true);
 
         /** Updates the catalog from POT file.
             \see Update
          */
-        bool UpdateFromPOT(const wxString& pot_file);
+        bool UpdateFromPOT(const wxString& pot_file, bool summary = true);
 
-        /** Adds translation into the catalog. 
+        /** Adds translation into the catalog.
             \return true on success or false if such key does
                      not exist in the catalog
          */
         bool Translate(const wxString& key, const wxString& translation);
 
-        /** Returns pointer to catalog item with key \a key or NULL if 
+        /** Returns pointer to catalog item with key \a key or NULL if
             such key is not available.
          */
         CatalogData* FindItem(const wxString& key) const;
@@ -190,6 +225,26 @@ class Catalog
         /// the object).
         void AddItem(CatalogData *data);
 
+        /// Adds entry to the catalog (the catalog will take ownership of
+        /// the object).
+        void AddDeletedItem(CatalogDeletedData *data);
+
+        /// Returns true if the catalog contains obsolete entries (~.*)
+        bool HasDeletedItems();
+
+        /// Removes all obsolete translations from the catalog
+        void RemoveDeletedItems();
+
+        /// Sets the given item to have the given bookmark and returns the index
+        /// of the item that previously had this bookmark (or -1)
+        int SetBookmark(int id, Bookmark bookmark);
+
+        /// Returns the index of the item that has the given bookmark or -1
+        int GetBookmarkIndex(Bookmark bookmark) const
+        {
+            return m_header.Bookmarks[bookmark];
+        }
+
     protected:
         /** Merges the catalog with reference catalog
             (in the sense of msgmerge -- this catalog is old one with
@@ -202,13 +257,13 @@ class Catalog
         bool Merge(Catalog *refcat);
 
         /** Returns list of strings that are new in reference catalog
-	        (compared to this one) and that are not present in \a refcat
+            (compared to this one) and that are not present in \a refcat
             (i.e. are obsoleted).
 
             \see ShowMergeSummary
          */
-	    void GetMergeSummary(Catalog *refcat, 
-	                         wxArrayString& snew, wxArrayString& sobsolete);
+        void GetMergeSummary(Catalog *refcat,
+                             wxArrayString& snew, wxArrayString& sobsolete);
 
         /** Shows a dialog with merge summary.
             \see GetMergeSummary, Merge
@@ -216,11 +271,12 @@ class Catalog
             \return true if the merge was OK'ed by the user, false otherwise
          */
         bool ShowMergeSummary(Catalog *refcat);
- 
+
     private:
         wxHashTable *m_data;
         CatalogDataArray m_dataArray;
-        unsigned m_count; // no. of items 
+        CatalogDeletedDataArray m_deletedItemsArray;
+        unsigned m_count; // no. of items
         bool m_isOk;
         wxString m_fileName;
         HeaderData m_header;
@@ -241,10 +297,10 @@ class CatalogParser
             new msgid/msgstr pair is found.
          */
         void Parse();
-            
+
     protected:
         /** Called when new entry was parsed. Parsing continues
-            if returned value is true and is cancelled if it 
+            if returned value is true and is cancelled if it
             is false.
          */
         virtual bool OnEntry(const wxString& msgid,
@@ -257,17 +313,30 @@ class CatalogParser
                              const wxArrayString& autocomments,
                              unsigned lineNumber) = 0;
 
+        /** Called when new deleted entry was parsed. Parsing continues
+            if returned value is true and is cancelled if it
+            is false. Defaults to an empty implementation.
+         */
+        virtual bool OnDeletedEntry(const wxArrayString& deletedLines,
+                                    const wxString& flags,
+                                    const wxArrayString& references,
+                                    const wxString& comment,
+                                    const wxArrayString& autocomments,
+                                    unsigned lineNumber)
+        {
+            return true;
+        }
+
         /// Textfile being parsed.
         wxTextFile *m_textFile;
         wxMBConv   *m_conv;
 };
 
-
 /** This class holds information about one particular string.
     This includes original string and its occurences in source code
     (so-called references), translation and translation's status
-    (fuzzy, non translated, translated and optional comment.
-    
+    (fuzzy, non translated, translated) and optional comment.
+
     This class is mostly internal, used by Catalog to store data.
  */
 class CatalogData : public wxObject
@@ -275,7 +344,7 @@ class CatalogData : public wxObject
     public:
         /// Ctor. Initializes the object with original string and translation.
         CatalogData(const wxString& str, const wxString& str_plural)
-                : wxObject(), 
+                : wxObject(),
                   m_string(str),
                   m_plural(str_plural),
                   m_hasPlural(false),
@@ -285,7 +354,8 @@ class CatalogData : public wxObject
                   m_isModified(false),
                   m_isAutomatic(false),
                   m_validity(Val_Unknown),
-                  m_lineNum(0) {}
+                  m_lineNum(0),
+                  m_bookmark(NO_BOOKMARK) {}
 
         CatalogData(const CatalogData& dt)
                 : wxObject(),
@@ -304,7 +374,8 @@ class CatalogData : public wxObject
                   m_comment(dt.m_comment),
                   m_validity(dt.m_validity),
                   m_lineNum(dt.m_lineNum),
-                  m_errorString(dt.m_errorString) {}
+                  m_errorString(dt.m_errorString),
+                  m_bookmark(dt.m_bookmark) {}
 
         /// Returns the original string.
         const wxString& GetString() const { return m_string; }
@@ -318,7 +389,7 @@ class CatalogData : public wxObject
         /// How many translations (plural forms) do we have?
         size_t GetNumberOfTranslations() const
             { return m_translations.GetCount(); }
- 
+
         /// Returns the nth-translation.
         wxString GetTranslation(unsigned n = 0) const;
 
@@ -337,7 +408,7 @@ class CatalogData : public wxObject
         /// Adds new reference to the entry (used by SourceDigger).
         void AddReference(const wxString& ref)
         {
-            if (m_references.Index(ref) == wxNOT_FOUND) 
+            if (m_references.Index(ref) == wxNOT_FOUND)
                 m_references.Add(ref);
         }
 
@@ -353,10 +424,10 @@ class CatalogData : public wxObject
             m_string = s;
             m_validity = Val_Unknown;
         }
-        
+
         /// Sets the plural form (if applicable).
-        void SetPluralString(const wxString& p) 
-        { 
+        void SetPluralString(const wxString& p)
+        {
             m_plural = p;
             m_hasPlural = true;
         }
@@ -364,7 +435,7 @@ class CatalogData : public wxObject
         /** Sets the translation. Changes "translated" status to true
             if \a t is not empty.
          */
-        void SetTranslation(const wxString& t, unsigned index = 0); 
+        void SetTranslation(const wxString& t, unsigned index = 0);
 
         /// Sets all translations.
         void SetTranslations(const wxArrayString& t);
@@ -375,8 +446,8 @@ class CatalogData : public wxObject
             m_comment = c;
         }
 
-        /** Sets gettext flags directly in string format. It may be 
-            either empty string or "#, fuzzy", "#, c-format", 
+        /** Sets gettext flags directly in string format. It may be
+            either empty string or "#, fuzzy", "#, c-format",
             "#, fuzzy, c-format" or others (not understood by poEdit).
          */
         void SetFlags(const wxString& flags);
@@ -408,12 +479,11 @@ class CatalogData : public wxObject
         /** Returns true if the gettext flags line contains "foo-format"
             flag when called with "foo" as argument. */
         bool IsInFormat(const wxString& format);
-        
+
         /// Adds new autocomments (#. )
         void AddAutoComments(const wxString& com)
         {
-            if (m_autocomments.Index(com) == wxNOT_FOUND) 
-                m_autocomments.Add(com);
+            m_autocomments.Add(com);
         }
 
         /// Clears autocomments.
@@ -421,14 +491,14 @@ class CatalogData : public wxObject
         {
             m_autocomments.Clear();
         }
-        
+
         // Validity (syntax-checking) status of the entry:
         enum Validity
         {
             Val_Unknown = -1,
             Val_Invalid = 0,
             Val_Valid = 1
-        };  
+        };
 
         /** Checks if %i etc. are correct in the translation (true if yes).
             Strings that are not c-format are always correct. */
@@ -439,11 +509,16 @@ class CatalogData : public wxObject
         void SetErrorString(const wxString& str) { m_errorString = str; }
         wxString GetErrorString() const { return m_errorString; }
 
+        /// Returns the bookmark for the item
+        Bookmark GetBookmark() {return m_bookmark;}
+        /// Returns true if the item has a bookmark
+        bool HasBookmark() {return (GetBookmark() != NO_BOOKMARK);}
+
     private:
         wxString m_string, m_plural;
         bool m_hasPlural;
         wxArrayString m_translations;
-        
+
         wxArrayString m_references, m_autocomments;
         bool m_isFuzzy, m_isTranslated, m_isModified, m_isAutomatic;
         bool m_hasBadTokens;
@@ -452,6 +527,111 @@ class CatalogData : public wxObject
         Validity m_validity;
         unsigned m_lineNum;
         wxString m_errorString;
+        Bookmark m_bookmark;
+
+        /// Sets the bookmark
+        void SetBookmark(Bookmark bookmark) {m_bookmark = bookmark;}
+
+   friend class Catalog;
+};
+
+/** This class holds information about one particular deleted item.
+    This includes deleted lines, references, translation's status
+    (fuzzy, non translated, translated) and optional comment(s).
+
+    This class is mostly internal, used by Catalog to store data.
+ */
+class CatalogDeletedData : public wxObject
+{
+    public:
+        /// Ctor. Initializes the object with original string and translation.
+        CatalogDeletedData(const wxArrayString& deletedLines)
+                : wxObject(),
+                  m_deletedLines(deletedLines),
+                  m_lineNum(0) {}
+
+        CatalogDeletedData(const CatalogDeletedData& dt)
+                : wxObject(),
+                  m_deletedLines(dt.m_deletedLines),
+                  m_references(dt.m_references),
+                  m_autocomments(dt.m_autocomments),
+                  m_flags(dt.m_flags),
+                  m_comment(dt.m_comment),
+                  m_lineNum(dt.m_lineNum) {}
+
+        /// Returns the deleted lines.
+        const wxArrayString& GetDeletedLines() const { return m_deletedLines; }
+
+        /// Returns array of all occurences of this string in source code.
+        const wxArrayString& GetReferences() const { return m_references; }
+
+        /// Returns comment added by the translator to this entry
+        const wxString& GetComment() const { return m_comment; }
+
+        /// Returns array of all auto comments.
+        const wxArrayString& GetAutoComments() const { return m_autocomments; }
+
+        /// Convenience function: does this entry has a comment?
+        const bool HasComment() const { return !m_comment.IsEmpty(); }
+
+        /// Adds new reference to the entry (used by SourceDigger).
+        void AddReference(const wxString& ref)
+        {
+            if (m_references.Index(ref) == wxNOT_FOUND)
+                m_references.Add(ref);
+        }
+
+        /// Clears references (used by SourceDigger).
+        void ClearReferences()
+        {
+            m_references.Clear();
+        }
+
+        /// Sets the string.
+        void SetDeletedLines(const wxArrayString& a)
+        {
+            m_deletedLines = a;
+        }
+
+        /// Sets the comment.
+        void SetComment(const wxString& c)
+        {
+            m_comment = c;
+        }
+
+        /** Sets gettext flags directly in string format. It may be
+            either empty string or "#, fuzzy", "#, c-format",
+            "#, fuzzy, c-format" or others (not understood by poEdit).
+         */
+        void SetFlags(const wxString& flags) {m_flags = flags;};
+
+        /// Gets gettext flags. \see SetFlags
+        wxString GetFlags() const {return m_flags;};
+
+        /// Sets the number of the line this entry occurs on.
+        void SetLineNumber(unsigned line) { m_lineNum = line; }
+        /// Get line number of this entry.
+        unsigned GetLineNumber() const { return m_lineNum; }
+
+        /// Adds new autocomments (#. )
+        void AddAutoComments(const wxString& com)
+        {
+            m_autocomments.Add(com);
+        }
+
+        /// Clears autocomments.
+        void ClearAutoComments()
+        {
+            m_autocomments.Clear();
+        }
+
+    private:
+        wxArrayString m_deletedLines;
+
+        wxArrayString m_references, m_autocomments;
+        wxString m_flags;
+        wxString m_comment;
+        unsigned m_lineNum;
 };
 
 #endif // _CATALOG_H_
