@@ -1,16 +1,31 @@
-
 /*
-
-    poedit, a wxWindows i18n catalogs editor
-
-    ---------------
-      chooselang.cpp
-    
-      Language chooser
-    
-      (c) Vaclav Slavik, 2003, 2004
-
-*/
+ *  This file is part of poEdit (http://www.poedit.org)
+ *
+ *  Copyright (C) 2003-2005 Vaclav Slavik
+ *
+ *  Permission is hereby granted, free of charge, to any person obtaining a
+ *  copy of this software and associated documentation files (the "Software"),
+ *  to deal in the Software without restriction, including without limitation
+ *  the rights to use, copy, modify, merge, publish, distribute, sublicense,
+ *  and/or sell copies of the Software, and to permit persons to whom the
+ *  Software is furnished to do so, subject to the following conditions:
+ *
+ *  The above copyright notice and this permission notice shall be included in
+ *  all copies or substantial portions of the Software.
+ *
+ *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+ *  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+ *  DEALINGS IN THE SOFTWARE.
+ *
+ *  $Id: chooselang.cpp,v 1.31 2005/06/19 16:43:20 vaclavslavik Exp $
+ *
+ *  Language chooser
+ *
+ */
 
 #include <wx/wxprec.h>
 
@@ -74,6 +89,7 @@ wxLanguage ChooseLanguage()
         { _T("Albanian"), wxLANGUAGE_ALBANIAN },
         { _T("Amharic"), wxLANGUAGE_AMHARIC },
         { _T("Bangla"), wxLANGUAGE_BENGALI },
+        { _T("Basque"), wxLANGUAGE_BASQUE },
         { _T("Belarusian"), wxLANGUAGE_BELARUSIAN },
         { _T("Breton"), wxLANGUAGE_BRETON },
         { _T("Bulgarian"), wxLANGUAGE_BULGARIAN },
@@ -88,15 +104,21 @@ wxLanguage ChooseLanguage()
         { _T("Estonian"), wxLANGUAGE_ESTONIAN },
         { _T("Esperanto"), wxLANGUAGE_ESPERANTO },
         { _T("Farsi"), wxLANGUAGE_FARSI },
+        { _T("Finnish"), wxLANGUAGE_FINNISH },
         { _T("French"), wxLANGUAGE_FRENCH },
+        { _T("Galician"), wxLANGUAGE_GALICIAN },
         { _T("Georgian"), wxLANGUAGE_GEORGIAN },
         { _T("German"), wxLANGUAGE_GERMAN },
         { _T("Greek"), wxLANGUAGE_GREEK },
+        { _T("Hebrew"), wxLANGUAGE_HEBREW },
         { _T("Hindi"), wxLANGUAGE_HINDI },
         { _T("Hungarian"), wxLANGUAGE_HUNGARIAN },
         { _T("Icelandic"), wxLANGUAGE_ICELANDIC },
+        { _T("Indonesian"), wxLANGUAGE_INDONESIAN },
         { _T("Italian"), wxLANGUAGE_ITALIAN },
         { _T("Japanese"), wxLANGUAGE_JAPANESE },
+        { _T("Korean"), wxLANGUAGE_KOREAN },
+        { _T("Kyrgyz"), wxLANGUAGE_KIRGHIZ },
         { _T("Latvian"), wxLANGUAGE_LATVIAN },
         { _T("Lithuanian"), wxLANGUAGE_LITHUANIAN },
         { _T("Mongolian"), wxLANGUAGE_MONGOLIAN },
@@ -112,9 +134,11 @@ wxLanguage ChooseLanguage()
         { _T("Slovak"), wxLANGUAGE_SLOVAK },
         { _T("Slovenian"), wxLANGUAGE_SLOVENIAN },
         { _T("Spanish"), wxLANGUAGE_SPANISH },
+        { _T("Spanish (Puerto Rico)"), wxLANGUAGE_SPANISH_PUERTO_RICO },
         { _T("Swedish"), wxLANGUAGE_SWEDISH },
         { _T("Turkish"), wxLANGUAGE_TURKISH },
         { _T("Tamil"), wxLANGUAGE_TAMIL },
+        { _T("Ukrainian"), wxLANGUAGE_UKRAINIAN },
 
         { NULL, wxLANGUAGE_UNKNOWN }
     };
