@@ -1,17 +1,31 @@
-
 /*
-
-    poedit, a wxWindows i18n catalogs editor
-
-    ---------------
-      edapp.cpp
-    
-      Application class
-    
-      (c) Vaclav Slavik, 1999-2004
-
-*/
-
+ *  This file is part of poEdit (http://www.poedit.org)
+ *
+ *  Copyright (C) 1999-2005 Vaclav Slavik
+ *
+ *  Permission is hereby granted, free of charge, to any person obtaining a
+ *  copy of this software and associated documentation files (the "Software"),
+ *  to deal in the Software without restriction, including without limitation
+ *  the rights to use, copy, modify, merge, publish, distribute, sublicense,
+ *  and/or sell copies of the Software, and to permit persons to whom the
+ *  Software is furnished to do so, subject to the following conditions:
+ *
+ *  The above copyright notice and this permission notice shall be included in
+ *  all copies or substantial portions of the Software.
+ *
+ *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+ *  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+ *  DEALINGS IN THE SOFTWARE.
+ *
+ *  $Id: edapp.cpp,v 1.51 2005/06/26 21:24:49 vaclavslavik Exp $
+ *
+ *  Application class
+ *
+ */
 
 #include <wx/wxprec.h>
 
@@ -59,7 +73,8 @@ wxString poEditApp::GetAppPath() const
 
 wxString poEditApp::GetAppVersion() const
 {
-    return _T("1.3.1");
+    wxString version(_T("1.3.3"));
+    return version;
 }
 
 
@@ -175,12 +190,36 @@ void poEditApp::SetDefaultParsers(wxConfigBase *cfg)
                                          _T("1.2.x"));
     pdb.Read(cfg);
 
-    // Add C/C++ parser (only if there's isn't any parser already):
-    if (pdb.GetCount() == 0)
+    // Add parsers for languages supported by gettext itself (but only if the
+    // user didn't already add language with this name himself):
+    static struct
     {
+        const wxChar *name;
+        const wxChar *exts;
+    } s_gettextLangs[] = {
+        { _T("C/C++"),    _T("*.c;*.cpp;*.h;*.hpp;*.cc;*.C;*.cxx;*.hxx") },
+#ifndef __WINDOWS__
+        // FIXME: not supported by 0.13.1 shipped with poedit on win32
+        { _T("C#"),       _T("*.cs") },
+#endif
+        { _T("Java"),     _T("*.java") },
+        { _T("Perl"),     _T("*.pl") },
+        { _T("PHP"),      _T("*.php") },
+        { _T("Python"),   _T("*.py") },
+        { _T("TCL"),      _T("*.tcl") },
+        { NULL, NULL }
+    };
+   
+    for (size_t i = 0; s_gettextLangs[i].name != NULL; i++)
+    {
+        // if this lang is already registered, don't overwrite it:
+        if (pdb.FindParser(s_gettextLangs[i].name) != -1)
+            continue;
+
+        // otherwise add new parser:
         Parser p;
-        p.Name = _T("C/C++");
-        p.Extensions = _T("*.c;*.cpp;*.h;*.hpp;*.cc;*.C;*.cxx;*.hxx");
+        p.Name = s_gettextLangs[i].name;
+        p.Extensions = s_gettextLangs[i].exts;
         p.Command = _T("xgettext --force-po -o %o %C %K %F");
         p.KeywordItem = _T("-k%k");
         p.FileItem = _T("%f");
@@ -207,17 +246,14 @@ void poEditApp::SetDefaultParsers(wxConfigBase *cfg)
     // If upgrading poEdit to 1.2.5, update C++ parser to handle --from-code:
     if (defaultsVersion == _T("1.2.x") || defaultsVersion == _T("1.2.4"))
     {
-        for (unsigned i = 0; i < pdb.GetCount(); i++)
+        int cpp = pdb.FindParser(_T("C/C++"));
+        if (cpp != -1)
         {
-            if (pdb[i].Name == _T("C/C++"))
+            if (pdb[cpp].Command == _T("xgettext --force-po -o %o %K %F"))
             {
-                if (pdb[i].Command == _T("xgettext --force-po -o %o %K %F"))
-                {
-                    pdb[i].Command = _T("xgettext --force-po -o %o %C %K %F");
-                    pdb[i].CharsetItem = _T("--from-code=%c");
-                    changed = true;
-                }
-                break;
+                pdb[cpp].Command = _T("xgettext --force-po -o %o %C %K %F");
+                pdb[cpp].CharsetItem = _T("--from-code=%c");
+                changed = true;
             }
         }
     }
