@@ -1,16 +1,31 @@
-
 /*
- 
-    poedit, a wxWindows i18n catalogs editor
- 
-    ---------------
-      gexecute.cpp
-    
-      Gettext execution code
-    
-      (c) Vaclav Slavik, 2000-2004
- 
-*/
+ *  This file is part of poEdit (http://www.poedit.org)
+ *
+ *  Copyright (C) 2000-2005 Vaclav Slavik
+ *
+ *  Permission is hereby granted, free of charge, to any person obtaining a
+ *  copy of this software and associated documentation files (the "Software"),
+ *  to deal in the Software without restriction, including without limitation
+ *  the rights to use, copy, modify, merge, publish, distribute, sublicense,
+ *  and/or sell copies of the Software, and to permit persons to whom the
+ *  Software is furnished to do so, subject to the following conditions:
+ *
+ *  The above copyright notice and this permission notice shall be included in
+ *  all copies or substantial portions of the Software.
+ *
+ *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+ *  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+ *  DEALINGS IN THE SOFTWARE.
+ *
+ *  $Id: gexecute.cpp,v 1.15 2005/06/19 16:43:21 vaclavslavik Exp $
+ *
+ *  Gettext execution code
+ *
+ */
 
 #include <wx/wxprec.h>
 
@@ -105,6 +120,8 @@ class TempLocaleSwitcher
 
 bool ExecuteGettext(const wxString& cmdline, wxString *stderrOutput)
 {
+    wxLogTrace(_T("poedit.execute"), _T("executing '%s'"), cmdline.c_str());
+
     TempLocaleSwitcher localeSwitcher(_T("C"));
 
     size_t i;
@@ -154,6 +171,8 @@ bool ExecuteGettextNonblocking(const wxString& cmdline,
                                GettextProcessData *data,
                                wxEvtHandler *parent)
 {
+    wxLogTrace(_T("poedit.execute"), _T("executing '%s'"), cmdline.c_str());
+
     TempLocaleSwitcher localeSwitcher(_T("C"));
 
     GettextProcess *process;
