@@ -1,16 +1,31 @@
-
 /*
-
-    poedit, a wxWindows i18n catalogs editor
-
-    ---------------
-      manager.cpp
-    
-      Catalogs manager
-    
-      (c) Vaclav Slavik, 2001,2004
-
-*/
+ *  This file is part of poEdit (http://www.poedit.org)
+ *
+ *  Copyright (C) 2001-2005 Vaclav Slavik
+ *
+ *  Permission is hereby granted, free of charge, to any person obtaining a
+ *  copy of this software and associated documentation files (the "Software"),
+ *  to deal in the Software without restriction, including without limitation
+ *  the rights to use, copy, modify, merge, publish, distribute, sublicense,
+ *  and/or sell copies of the Software, and to permit persons to whom the
+ *  Software is furnished to do so, subject to the following conditions:
+ *
+ *  The above copyright notice and this permission notice shall be included in
+ *  all copies or substantial portions of the Software.
+ *
+ *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+ *  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+ *  DEALINGS IN THE SOFTWARE.
+ *
+ *  $Id: manager.cpp,v 1.23 2005/06/19 16:43:21 vaclavslavik Exp $
+ *
+ *  Catalogs manager frame
+ *
+ */
 
 #include <wx/wxprec.h>
 
@@ -23,7 +38,7 @@
 #include <wx/intl.h>
 #include <wx/frame.h>
 #include <wx/listbox.h>
-#include <wx/gizmos/editlbox.h>
+#include "editlbox/editlbox.h"
 #include <wx/tokenzr.h>
 #include <wx/msgdlg.h>
 #include <wx/button.h>
@@ -209,7 +224,7 @@ void ManagerFrame::UpdateListCat(int id)
     m_catalogs.Clear();
     while (tkn.HasMoreTokens())
         wxDir::GetAllFiles(tkn.GetNextToken(), &m_catalogs, 
-                           _T("*.po"), wxDIR_FILES);
+                           _T("*.po"), wxDIR_FILES | wxDIR_DIRS);
 
     m_listCat->Freeze();
 
@@ -366,7 +381,7 @@ void ManagerFrame::OnEditProject(wxCommandEvent& event)
 {
 	int sel = m_listPrj->GetSelection();
 	if (sel == -1) return;
-    EditProject((int)m_listPrj->GetClientData(sel));
+    EditProject((long)m_listPrj->GetClientData(sel));
 }
 
 
@@ -376,7 +391,7 @@ void ManagerFrame::OnDeleteProject(wxCommandEvent& event)
 	if (sel == -1) return;
     if (wxMessageBox(_("Do you want to delete the project?"),
                _("Confirmation"), wxYES_NO | wxICON_QUESTION, this) == wxYES)
-        DeleteProject((int)m_listPrj->GetClientData(sel));
+        DeleteProject((long)m_listPrj->GetClientData(sel));
 }
 
 
@@ -384,7 +399,7 @@ void ManagerFrame::OnSelectProject(wxCommandEvent& event)
 {
 	int sel = m_listPrj->GetSelection();
 	if (sel == -1) return;
-    m_curPrj = (int)m_listPrj->GetClientData(sel);
+    m_curPrj = (long)m_listPrj->GetClientData(sel);
     UpdateListCat(m_curPrj);
 }
 
