@@ -1,16 +1,31 @@
-
 /*
-
-    poedit, a wxWindows i18n catalogs editor
-
-    ---------------
-      transmem.cpp
-    
-      Translation memory database
-    
-      (c) Vaclav Slavik, 2001-2003
-
-*/
+ *  This file is part of poEdit (http://www.poedit.org)
+ *
+ *  Copyright (C) 2001-2005 Vaclav Slavik
+ *
+ *  Permission is hereby granted, free of charge, to any person obtaining a
+ *  copy of this software and associated documentation files (the "Software"),
+ *  to deal in the Software without restriction, including without limitation
+ *  the rights to use, copy, modify, merge, publish, distribute, sublicense,
+ *  and/or sell copies of the Software, and to permit persons to whom the
+ *  Software is furnished to do so, subject to the following conditions:
+ *
+ *  The above copyright notice and this permission notice shall be included in
+ *  all copies or substantial portions of the Software.
+ *
+ *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+ *  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+ *  DEALINGS IN THE SOFTWARE.
+ *
+ *  $Id: transmem.cpp,v 1.21 2005/06/19 16:43:21 vaclavslavik Exp $
+ *
+ *  Translation memory database
+ *
+ */
 
 #include <wx/wxprec.h>
 
@@ -379,7 +394,7 @@ wxArrayString *DbTrans::Read(DbKey index)
 
 DbKey DbOrig::Read(const wxString& str)
 {
-    const wxWX2MBbuf c_str_buf = str.mb_str();
+    const wxWX2MBbuf c_str_buf = str.mb_str(wxConvUTF8);
     const char *c_str = c_str_buf;
     DBT key, data;
 
@@ -402,7 +417,7 @@ DbKey DbOrig::Read(const wxString& str)
 
 bool DbOrig::Write(const wxString& str, DbKey value)
 {
-    const wxWX2MBbuf c_str_buf = str.mb_str();
+    const wxWX2MBbuf c_str_buf = str.mb_str(wxConvUTF8);
     const char *c_str = c_str_buf;
     DBT key, data;
 
@@ -427,7 +442,7 @@ bool DbOrig::Write(const wxString& str, DbKey value)
 
 DbKeys *DbWords::Read(const wxString& word, unsigned sentenceSize)
 {
-    const wxWX2MBbuf word_mb = word.mb_str();
+    const wxWX2MBbuf word_mb = word.mb_str(wxConvUTF8);
     char *keyBuf;
     size_t keyLen;
     DBT key, data;
@@ -465,7 +480,7 @@ bool DbWords::Append(const wxString& word, unsigned sentenceSize, DbKey value)
     //     This is important because it allows us to efficiently merge
     //     these lists when looking up inexact translations...
 
-    const wxWX2MBbuf word_mb = word.mb_str();
+    const wxWX2MBbuf word_mb = word.mb_str(wxConvUTF8);
     DbKey *valueBuf;
     char *keyBuf;
     size_t keyLen;
